(defconst parse-time-rfc2822-regex
  (let* ((fws "[ \t\r\n]+")
         (ofws "[ \t\r\n]*")
         (day-of-week
          (regexp-opt
           (mapcar 'car parse-time-weekdays)))
         (day "[0-9]\\{1,2\\}")
         (month
          (regexp-opt
           (mapcar 'car parse-time-months)))
         (year "[0-9]\\{4,\\}")
         (hour "[0-9]\\{2\\}")
         (minute hour)
         (second hour)
         (zone-hour (concat "[+-]" hour))
         (zone-min hour)
         ;; A rather non strict comment.
         (cfws "(\\(?:.\\|\n\\)*)"))
    (concat
     (format "\\`%s\\(?:\\(%s\\),%s\\)?" ofws day-of-week ofws)
     (format "\\(%s\\)%s\\(%s\\)%s\\(%s\\)%s"
             day fws month fws year fws)
     (format "\\(%s\\):\\(%s\\)\\(?::\\(%s\\)\\)?%s"
             hour minute second fws)
     (format "\\(%s\\)\\(%s\\)" zone-hour zone-min)
     (format "%s\\(?:%s\\)?%s\\'" ofws cfws ofws)))
  "A regex matching a strict rfc2822 date.")

(defun parse-time-string-rfc2822 (string)
  "Parse the strict rfc2822 time-string STRING

Return either a list (SEC MIN HOUR DAY MON YEAR DOW DST TZ) or
nil, if STRING is not valid."
  (when (string-match parse-time-rfc2822-regex string)
    (let ((dow (cdr (assoc (match-string 1 string)
                           parse-time-weekdays)))
          (day (string-to-number (match-string 2 string)))
          (month (cdr (assoc (match-string 3 string)
                             parse-time-months)))
          (year (string-to-number (match-string 4 string)))
          (hour (string-to-number (match-string 5 string)))
          (min (string-to-number (match-string 6 string)))
          (sec (string-to-number (or (match-string 7 string) "0")))
          (zhour (string-to-number (match-string 8 string)))
          (zmin (string-to-number (match-string 9 string))))
      (list sec min hour day month year dow nil (+ (* 3600 zhour)
                                                   (* 60 zmin))))))
